"""
S1 Pattern Maps

https://xe1t-wiki.lngs.infn.it/doku.php?id=min:s1opticalmap_comparison (Min + Giovanni)

"""

from ..base_references import BaseMap


class S1PatternMap(BaseMap):
    _ALIAS = "s1_pattern_maps"
    fmt = "pkl"
